from typing import Any, Optional

from octostar_streamlit.core.entities import WorkspaceItem
from octostar_streamlit.core.saved_search.method_call import SavedSearchMethodCall, SavedSearchApiMethod
from octostar_streamlit.core.saved_search.params import SavedSearchIdn
from octostar_streamlit.core.streamlit_executor import StreamlitMethodExecutor
from octostar_streamlit import _component_func


def call_saved_search_api_method(method: SavedSearchApiMethod, params: Any, key=None):
    call = SavedSearchMethodCall(service="savedSearch", method=method, params=params)
    value = StreamlitMethodExecutor(
        method_call=call, fn=_component_func, key=key
    ).execute()

    return value


def get_records_count(params: SavedSearchIdn, key=None) -> Optional[WorkspaceItem]:
    return call_saved_search_api_method("getRecordsCount", params, key=key)



from typing import Any, Optional

from octostar_streamlit.core.entities import WorkspaceItem
from octostar_streamlit.core.extras.method_call import ExtrasApiMethod, ExtrasMethodCall
from octostar_streamlit.core.extras.params import CreateLinkChartParams
from octostar_streamlit.core.streamlit_executor import StreamlitMethodExecutor
from octostar_streamlit import _component_func


def call_extras_api_method(method: ExtrasApiMethod, params: Any, key=None):
    call = ExtrasMethodCall(service="extras", method=method, params=params)
    value = StreamlitMethodExecutor(
        method_call=call, fn=_component_func, key=key
    ).execute()

    return value


def create_link_chart(params: CreateLinkChartParams, key=None) -> Optional[WorkspaceItem]:
    return call_extras_api_method("createLinkChart", params, key=key)



from typing import Callable
from uuid import uuid4

from pydantic import BaseModel
from octostar_streamlit.core.method_call import MethodCall


class StreamlitMethodExecutor:
    def __init__(self, method_call: MethodCall, fn: Callable, key=None, subscribe=None) -> None:
        self._method_call = method_call
        self._fn = fn
        self._key = key
        self._subscribe = subscribe

    def execute(self):
        key = self._key
        service = self._method_call.service
        method = self._method_call.method

        if isinstance(self._method_call.params, BaseModel):
            # params = self._method_call.params.model_dump(by_alias=True)
            params = self._method_call.params.model_dump()
        else:
            params = self._method_call.params

        return self._fn(service=service, method=method, params=params, key=key, subscribe=self._subscribe)



from typing import Any, Dict, List, Union

from altair import param
from octostar_streamlit.core.entities import (
    Concept,
    Entity,
    Inheritance,
    Relationship,
    RelationshipCountResult,
    WorkspaceRelationship,
)
from octostar_streamlit.core.ontology.method_call import (
    OntologyApiMethod,
    OntologyMethodCall,
)
from octostar_streamlit.core.ontology.params import (
    CancelQueriesParams,
    ClearRelationshipCacheParams,
    ConsistentUUIDParams,
    GetConceptByNameParams,
    GetConceptForEntityparams,
    GetConnectedEntitiesParams,
    GetEntityParams,
    GetRelationshipCountParams,
    GetRelationshipsForEntityParams,
    GetWorkspaceRelationshipRecordsParams,
    SendQueryParams,
    SendQueryTParams,
)
from octostar_streamlit.core.streamlit_executor import StreamlitMethodExecutor
from octostar_streamlit import _component_func


def call_ontology_api_method(method: OntologyApiMethod, params: Any, key=None):
    method_call = OntologyMethodCall(service="ontology", method=method, params=params)
    value = StreamlitMethodExecutor(
        method_call=method_call, fn=_component_func, key=key
    ).execute()
    return value


def cancel_queries(params: CancelQueriesParams, key=None) -> None:
    return call_ontology_api_method("cancelQueries", params, key)


def get_available_ontologies(key=None) -> List[str]:
    return call_ontology_api_method("getAvailableOntologies", None, key)


def get_workspace_relationship_records(
    params: GetWorkspaceRelationshipRecordsParams, key=None
) -> List[WorkspaceRelationship]:
    result = call_ontology_api_method("getWorkspaceRelationshipRecords", params, key)
    return [WorkspaceRelationship(**record) for record in result]


def clear_relationship_cache(params: ClearRelationshipCacheParams, key=None) -> None:
    return call_ontology_api_method("clearRelationshipCache", params, key)


def get_connected_entities(
    params: GetConnectedEntitiesParams, key=None
) -> List[Entity]:
    result = call_ontology_api_method("getConnectedEntities", params, key)
    return [Entity(**entity) for entity in result]


def get_concept_by_name(
    params: GetConceptByNameParams, key=None
) -> Union[Concept, None]:
    result = call_ontology_api_method("getConceptByName", params, key)
    if result is None:
        return None
    return Concept(**result)


def get_concepts(key=None) -> Dict[str, Concept]:
    result = call_ontology_api_method("getConcepts", None, key)
    return {key: Concept(**value) for key, value in result.items()}


def get_entity(params: GetEntityParams, key=None) -> Entity:
    result = call_ontology_api_method("getEntity", params, key)
    return Entity(**result)


def get_entity_by_id(entity_id: str, entity_type: str, refresh: bool = False, key=None) -> Entity:
    params = GetEntityParams(entity=Entity(entity_id=entity_id, entity_type=entity_type, entity_label=''), refresh=refresh)
    return get_entity(params=params, key=key)

def get_ontology_name(key=None) -> str:
    return call_ontology_api_method("getOntologyName", None, key)


def get_relationship_count(
    params: GetRelationshipCountParams, key=None
) -> RelationshipCountResult:
    result = call_ontology_api_method("getRelationshipCount", params, key)
    return RelationshipCountResult(**result)


def get_concept_for_entity(
    params: GetConceptForEntityparams, key=None
) -> Union[Concept, None]:
    result = call_ontology_api_method("getConceptForEntity", params, key)
    if result is None:
        return None
    return Concept(**result)


def get_relationships_for_entity(
    params: GetRelationshipsForEntityParams, key=None
) -> List[Relationship]:
    result = call_ontology_api_method("getRelationshipsForEntity", params, key)
    return [Relationship(**relationship) for relationship in result]


def send_query(params: SendQueryParams, key=None) -> List[Any]:
    return call_ontology_api_method("sendQuery", params, key)


def send_query_t(params: SendQueryTParams, key=None) -> List[Any]:
    return call_ontology_api_method("sendQueryT", params, key)

def get_sys_inheritance(key=None) -> List[Inheritance]:
    result = call_ontology_api_method("getSysInheritance", None, key)
    return [Inheritance(**inheritance) for inheritance in result]


# TODO: implement subscribe


def consistent_uuid(params: ConsistentUUIDParams, key=None) -> str:
    return call_ontology_api_method("consistentUUID", params, key)



from typing import Dict, Union
from octostar_streamlit.core.params_base_model import ParamsBaseModel

class OsDropzoneParams(ParamsBaseModel):
    label: str

class OsContextMenuParams(ParamsBaseModel):
    item: Dict
    label: Union[str, None] = None
    height: Union[str, None] = None
    padding: Union[str, None] = None



from dataclasses import dataclass
from io import FileIO
from typing import Any, Callable, Dict, List, Literal, Union, Optional

from pydantic import ConfigDict, Field
from octostar_streamlit.core.entities import (
    AttachmentResponseType,
    AttachmentType,
    ButtonProps,
    ContextMenuGroup,
    ContextMenuLabels,
    Entity,
    GraphSpec,
    IDataTransfer,
    ItemType,
    KeyValueTuple,
    OsTag,
    Relationship,
    TagAttributes,
    Workspace,
    WorkspaceItem,
    WorkspaceRecordIdentifier,
    WorkspaceRecordWithRelationships,
)
from octostar_streamlit.core.params_base_model import ParamsBaseModel


class GetActiveWorkspaceParams(ParamsBaseModel):
    prompt: bool = False


BinaryOperator = Literal['==', '!=', '>', '>=', '<', '<=', 'in', 'not in', 'regex', 'like', 'ilike']
Comparator = Union[str, int, List[Union[str, int]], None]

class QueryObjectFilterClause(ParamsBaseModel):
    col: str
    op: BinaryOperator
    val: Comparator

class ConceptFilter(ParamsBaseModel):
    concept: str
    filters: List[QueryObjectFilterClause]

class GetPastContextParams(ParamsBaseModel):
    limit1: Union[int, None]
    limit2: Union[int, None]
    record: Union[Entity, None]
    records: Union[List[Entity], None]
    query: Union[ConceptFilter, None] = None
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        if self.records is not None:
            base_dict['records'] = [record.to_dict() for record in self.records]
        if self.record is not None:
            base_dict['record'] = self.record.to_dict()
        return base_dict

class CopyParams(ParamsBaseModel):
    source: WorkspaceItem
    target: WorkspaceItem


@dataclass
class GetAttachmentParamsOptions:
    path: Union[str, None] = None
    default: Union[AttachmentType, None] = None
    responseType: Union[AttachmentResponseType, None] = None


class GetAttachmentParams(ParamsBaseModel):
    entity: Entity
    options: Union[GetAttachmentParamsOptions, None] = None
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict

class GetStylerParams(ParamsBaseModel):
    name: str
    context: Dict


class GetWorkspaceItemsParams(ParamsBaseModel):
    os_item_name: str


class ApplyTagParams(ParamsBaseModel):
    os_workspace: str
    tag: Union[OsTag, TagAttributes]
    entity: Union[Entity, List[Entity]]
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        if isinstance(self.entity, list):
            base_dict['entity'] = [record.to_dict() for record in self.entity]
        else:
            base_dict['entity'] = self.entity.to_dict()
        return base_dict

class RemoveTagParams(ApplyTagParams):
    pass


class UpdateTagParams(ParamsBaseModel):
    tag: OsTag


class GetTagsParams(Entity, ParamsBaseModel):
    pass


class GetAvailableTagsParams(ParamsBaseModel):
    entity: Entity
    workspace: Union[str, None] = None
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict

class GetTemplateParams(ParamsBaseModel):
    name: str
    defaultTemplate: Union[str, None] = None


class GetSchemaItemsParams(ParamsBaseModel):
    os_item_content_type: str


class CreateWorkspaceParams(ParamsBaseModel):
    name: str


class ConnectParams(ParamsBaseModel):
    relationship: Union[Relationship, str]
    from_entity: Entity
    to_entity: Entity
    os_workspace: Union[str, None] = None
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['from_entity'] = self.from_entity.to_dict()
        base_dict['to_entity'] = self.to_entity.to_dict()
        return base_dict


@dataclass
class SaveFileOptions:
    path: Union[str, None] = None


@dataclass
class SaveParamsOptions(SaveFileOptions):
    draft: Union[bool, None] = None
    saveAs: Union[bool, None] = None
    app: Union[WorkspaceItem, None] = None


class SaveParams(ParamsBaseModel):
    item: Union[
        WorkspaceItem, WorkspaceRecordIdentifier, WorkspaceRecordWithRelationships
    ]
    options: Union[SaveParamsOptions, None] = None


class SaveFileParams(ParamsBaseModel):
    item: WorkspaceItem
    file: Union[FileIO, str]
    options: Union[SaveFileOptions, None] = None


class ImportItemsParams(ParamsBaseModel):
    items: List[Any]


@dataclass
class ImportZipOptions:
    overwrite: Union[bool, None] = None
    target: Union[str, None] = None


class ImportZipParams(ParamsBaseModel):
    file: FileIO
    options: Union[ImportZipOptions, None] = None


@dataclass
class ExportOptions:
    filename: Union[str, None] = None


class ExportParams(ParamsBaseModel):
    item: Union[WorkspaceItem, List[WorkspaceItem]]
    options: Union[ExportOptions, None] = None


@dataclass
class FileTreeOptions:
    recurse: Union[bool, None] = None
    exclude_root: Union[bool, None] = None
    flat: Union[bool, None] = None
    minimal: Union[bool, None] = None
    structure: Union[bool, None] = None


class GetFilesTreeParams(ParamsBaseModel):
    workspace_or_folder: WorkspaceItem
    options: Union[FileTreeOptions, None] = None


class DesktopActionOptions(ParamsBaseModel):
    besideTabId: Union[str, None] = None
    autoSelectTab: Union[bool, None] = None
    onCloseTab: Union[Callable, None] = None
    desktopWith: Union[str, None] = Field(default=None, alias='with')
    initialState: Union[Any, None] = None


class OpenParams(ParamsBaseModel):
    records: Union[Entity, List[Entity]]
    options: Union[DesktopActionOptions, None] = None

    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        if isinstance(self.records, list):
            base_dict['records'] = [record.to_dict() for record in self.records]
        else:
            base_dict['records'] = self.records.to_dict()
        return base_dict


class DeleteParams(ParamsBaseModel):
    item: Union[WorkspaceRecordIdentifier, List[WorkspaceRecordIdentifier]]
    recurse: Union[bool, None] = None


@dataclass
class SearchXperienceOptionsDefaultSearchFields:
    entity_label: Union[str, None] = None
    os_textsearchfield: Union[str, None] = None


class SearchXperienceParams(ParamsBaseModel):
    taskID: Union[str, None] = None
    title: Union[str, None] = None
    defaultConcept: Union[List[str], None] = None
    disableConceptSelector: Union[bool, None] = None
    defaultSearchFields: Union[SearchXperienceOptionsDefaultSearchFields, None] = None

class BaseEntityModel(ParamsBaseModel):
    entity_id: str
    entity_type: str
    entity_label: str
    model_config = ConfigDict(extra="allow") # Permits additional fields



class OsAuditFieldsModel(ParamsBaseModel):
    os_created_by: Optional[str] = None
    os_created_at: Optional[str] = None
    os_deleted_by: Optional[str] = None
    os_deleted_at: Optional[str] = None
    os_hidden_by: Optional[str] = None
    os_hidden_at: Optional[str] = None
    os_last_updated_by: Optional[str] = None
    os_last_updated_at: Optional[str] = None


class WorkspaceItemBase(ParamsBaseModel):
    os_entity_uid: str
    os_workspace: str


class WorkspaceRecord(BaseEntityModel, WorkspaceItemBase, OsAuditFieldsModel):
    os_icon: Optional[str] = None


class WorkspaceItemModel(WorkspaceRecord):
    os_item_content: Optional[dict] = None
    os_item_content_type: Optional[str] = None
    os_parent_folder: Optional[str] = None
    os_has_attachment: Optional[bool] = None
    os_path: Optional[str] = None


class SearchFilter(ParamsBaseModel):
    label: Optional[str] = None
    entity: Optional[BaseEntityModel] = None
    query: Optional[Dict] = None # Elasticsearch QueryDslQueryContainer
    imageUrl: Optional[str] = None


class SearchResultsParams(ParamsBaseModel):
    q: Optional[str] = None
    submitButtonLabel: Optional[str] = None
    filters: Optional[List[SearchFilter]] = None
    image: Optional[WorkspaceItemModel] = None



class ShowTabParams(ParamsBaseModel):
    app: WorkspaceItem
    item: Union[WorkspaceItem, None] = None
    options: Union[DesktopActionOptions, None] = None


class CloseTabParams(ShowTabParams):
    pass


class CallAppServiceParams(ParamsBaseModel):
    service: str
    context: Dict
    options: Union[Dict, None] = None


@dataclass
class ActionContext:
    concept: Union[str, None] = None
    # Probably: Union[Union[GraphSpec, Any], None]
    graph: Union[GraphSpec, None] = None
    item: Union[WorkspaceItem, None] = None
    items: Union[List[WorkspaceItem], None] = None
    workspace: Union[Workspace, None] = None
    dataTransfer: Union[IDataTransfer, None] = None
    eventTopicPrefix: Union[str, None] = None
    then: Union[Callable, None] = None


@dataclass
class ContextMenuOptions:
    mode: Union[Literal["edit", "default"], None] = None
    groups: Union[List[ContextMenuGroup], None] = None
    labels: Union[ContextMenuLabels, None] = None
    extras: Union[List[ItemType], None] = None


class OpenContextMenuParams(ActionContext, ParamsBaseModel):
    x: int
    y: int
    openContextMenu: Union[bool, None] = None
    onCloseEmit: Union[str, None] = None
    options: Union[ContextMenuOptions, None] = None


class CloseContextMenuParams(ParamsBaseModel):
    clearContextMenu: Literal[True]


# OsNotification
class ShowToastParams(ParamsBaseModel):
    message: str
    id: Union[str, None] = None
    description: Union[str, None] = None
    level: Union[Literal["info", "success", "error", "warning"], None] = None
    placement: Union[
        Literal["top", "bottom", "topLeft", "topRight", "bottomLeft", "bottomRight"],
        None,
    ] = None


class ClearToastParams(ParamsBaseModel):
    id: str


class ShowProgressParams(ParamsBaseModel):
    key: str
    label: Union[str, None] = None
    job_type: Union[str, None] = None
    status: Union[Literal["active", "success", "exception", "normal"], None] = None


class ShowConfirmParams(ParamsBaseModel):
    title: str
    icon: Union[str, None] = None
    content: Union[str, None] = None
    okText: Union[str, None] = None
    okButtonProps: Union[ButtonProps, None] = None
    cancelText: Union[str, None] = None
    taskID: Union[str, None] = None


class ShowCreateEntityFormParams(ParamsBaseModel):
    os_workspace: str
    concept: Union[str, None] = None


@dataclass
class AddCommentParamsComment:
    os_workspace: str
    contents: str
    os_parent_uid: Union[str, None] = None
    slug: Union[str, None] = None


class AddCommentParams(ParamsBaseModel):
    about: Entity
    comment: AddCommentParamsComment
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['about'] = self.about.to_dict()
        return base_dict

class RemoveCommentParams(ParamsBaseModel):
    os_workspace: str
    comment_id: str


class Watcher(ParamsBaseModel):
    app_id: str
    app_name: str
    watcher_name: str
    const: List[KeyValueTuple]
    description: str
    file: str
    interval: str
    name: str
    params: List[KeyValueTuple]
    semantically_bound: List[str]


class AddWatchIntentParams(ParamsBaseModel):
    entity: Entity
    watcher: Watcher
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict

class RemoveWatchIntentParams(ParamsBaseModel):
    os_workspace: str
    intent_id: str


class GetWorkspacePermissionParams(ParamsBaseModel):
    os_workspace: List[str]


class DeployAppParams(ParamsBaseModel):
    app: WorkspaceItem


class UndeployAppParams(ParamsBaseModel):
    app: WorkspaceItem



from typing import Any, Literal

from octostar_streamlit.core.method_call import MethodCall


ContextApiMethod = Literal["getContext", "subscribeToChanges", "unsubscribeFromChanges"]


class ContextMethodCall(MethodCall):
    service: Literal["context"] = "context"
    method: ContextApiMethod
    params: Any



from typing import Any, Dict, List, Union, Optional
from octostar_streamlit.core.desktop.params import (
    AddCommentParams,
    AddWatchIntentParams,
    ApplyTagParams,
    CallAppServiceParams,
    ClearToastParams,
    CloseContextMenuParams,
    CloseTabParams,
    ConnectParams,
    CopyParams,
    CreateWorkspaceParams,
    DeleteParams,
    DeployAppParams,
    ExportParams,
    GetActiveWorkspaceParams,
    GetAttachmentParams,
    GetAvailableTagsParams,
    GetFilesTreeParams,
    GetPastContextParams,
    GetSchemaItemsParams,
    GetStylerParams,
    GetTagsParams,
    GetTemplateParams,
    GetWorkspaceItemsParams,
    GetWorkspacePermissionParams,
    ImportItemsParams,
    ImportZipParams,
    OpenContextMenuParams,
    OpenParams,
    RemoveCommentParams,
    RemoveTagParams,
    RemoveWatchIntentParams,
    SaveFileParams,
    SaveParams,
    SearchResultsParams,
    SearchXperienceParams,
    ShowConfirmParams,
    ShowCreateEntityFormParams,
    ShowProgressParams,
    ShowTabParams,
    ShowToastParams,
    UndeployAppParams,
    UpdateTagParams,
)
from octostar_streamlit.core.entities import (
    AttachmentType,
    CustomTemplate,
    Entity,
    OsTag,
    OsWorkspaceEntity,
    Styler,
    StylerOption,
    UserProfile,
    Whoami,
    Workspace,
    WorkspaceIdHost,
    WorkspaceItem,
    WorkspacePermission,
)
from octostar_streamlit.core.desktop.method_call import (
    DesktopApiMethod,
    DesktopMethodCall,
)
from octostar_streamlit.core.streamlit_executor import StreamlitMethodExecutor
from octostar_streamlit import _component_func


# TODO: provide a way to specify params as optional or required
def call_desktop_api_method(method: DesktopApiMethod, params: Any, key=None, subscribe=None) -> Any:
    call = DesktopMethodCall(service="desktop", method=method, params=params)
    value = StreamlitMethodExecutor(
        method_call=call, fn=_component_func, key=key, subscribe=subscribe
    ).execute()
    return value


def refresh(key=None) -> None:
    return call_desktop_api_method("refresh", None, key)


# def open_workspace(params: OpenWorkspaceParams, key=None) -> None:
def open_workspace(params: Union[WorkspaceIdHost, WorkspaceItem], key=None) -> None:
    return call_desktop_api_method("openWorkspace", params, key)

def close_workspace(params: Union[WorkspaceIdHost, WorkspaceItem], key=None) -> None:
    return call_desktop_api_method("closeWorkspace", params, key)

# TODO: Don't see anyone using this, need examples to test it
def get_paste_context(params: GetPastContextParams, key=None) -> List[Entity]:
    return call_desktop_api_method("getPasteContext", params, key)

def get_active_workspace(
    params: GetActiveWorkspaceParams=GetActiveWorkspaceParams(), key=None
) -> Union[str, None]:
    return call_desktop_api_method("getActiveWorkspace", params, key)

def set_active_workspace(params: WorkspaceIdHost, key=None) -> None:
    return call_desktop_api_method("setActiveWorkspace", params, key)

def copy(params: CopyParams, key=None) -> None:
    return call_desktop_api_method("copy", params, key)


def list_workspaces(key=None) -> List[WorkspaceItem]:
    return call_desktop_api_method("listAllWorkspaces", None, key)

def get_open_workspaces(key=None) -> Optional[List[Workspace]]:
    result = call_desktop_api_method("getOpenWorkspaces", None, key, subscribe="onOpenWorkspacesChanged")
    if result is None:
        return None
    return [Workspace(**v) for v in result]

def get_open_workspace_ids(key=None) -> Optional[List[str]]:
    result = call_desktop_api_method("getOpenWorkspaceIds", None, key, subscribe="onOpenWorkspaceIdsChanged")
    return result

def on_workspace_changed(uuid: str, key=None) -> Optional[Workspace]:
    result = call_desktop_api_method("onWorkspaceChanged", uuid, key)
    if result is None:
        return None
    return Workspace(**result)

def on_workspace_item_changed(uuid: str, key=None) -> Optional[Entity]:
    result = call_desktop_api_method("onWorkspaceItemChanged", uuid, key)
    if result is None:
        return None
    return Entity(**result)

def set_open_workspace_ids(ids: List[str], key=None) -> None:
    call_desktop_api_method("setOpenWorkspaceIds", ids, key)

def get_attachment(params: GetAttachmentParams, key=None) -> AttachmentType:
    return call_desktop_api_method("getAttachment", params, key)


def get_styler_options(key=None) -> Optional[List[StylerOption]]:
    result = call_desktop_api_method("getStylerOptions", None, key)
    if result is None:
        return None
    return [StylerOption(**v) for v in result]


def get_styler(params: GetStylerParams, key=None) -> Styler:
    return call_desktop_api_method("getStyler", params, key)


def get_workspace(
    params: Union[WorkspaceIdHost, WorkspaceItem], key=None
) -> Optional[Workspace]:
    result = call_desktop_api_method("getWorkspace", params, key)
    if result is None:
        return None
    return Workspace(**result)


def get_workspace_items(
    params: GetWorkspaceItemsParams, key=None
) -> List[WorkspaceItem]:
    return call_desktop_api_method("getWorkspaceItems", params, key)


def apply_tag(params: ApplyTagParams, key=None) -> None:
    return call_desktop_api_method("applyTag", params, key)


def remove_tag(params: RemoveTagParams, key=None) -> None:
    return call_desktop_api_method("removeTag", params, key)


def update_tag(params: UpdateTagParams, key=None) -> OsTag:
    return call_desktop_api_method("updateTag", params, key)


def get_item(params: Union[WorkspaceIdHost, WorkspaceItem], key=None) -> WorkspaceItem:
    return call_desktop_api_method("getItem", params, key)


def get_items(
    params: List[Union[WorkspaceIdHost, WorkspaceItem]], key=None
) -> List[WorkspaceItem]:
    return call_desktop_api_method("getItems", params, key)


def get_tags(params: GetTagsParams, key=None) -> List[Dict]:
    return call_desktop_api_method("getTags", params, key)


def get_available_tags(params: GetAvailableTagsParams, key=None) -> List[OsTag]:
    return call_desktop_api_method("getAvailableTags", params, key)


def get_templates(key=None) -> Optional[List[CustomTemplate]]:
    result = call_desktop_api_method("getTemplates", None, key)
    if result is None:
        return None
    return [CustomTemplate(**v) for v in result]


def get_template(params: GetTemplateParams, key=None) -> Optional[CustomTemplate]:
    result = call_desktop_api_method("getTemplate", params, key)
    if result is None:
        return None
    return CustomTemplate(**result)


def get_schema_items(params: GetSchemaItemsParams, key=None) -> List[WorkspaceItem]:
    return call_desktop_api_method("getSchemaItems", params, key)


def create_workspace(params: CreateWorkspaceParams, key=None) -> WorkspaceItem:
    return call_desktop_api_method("createWorkspace", params, key)


def connect(params: ConnectParams, key=None) -> WorkspaceItem:
    return call_desktop_api_method("connect", params, key)


def save(params: SaveParams, key=None) -> WorkspaceItem:
    return call_desktop_api_method("save", params, key)


def save_file(params: SaveFileParams, key=None) -> WorkspaceItem:
    return call_desktop_api_method("saveFile", params, key)


def import_items(params: ImportItemsParams, key=None) -> None:
    return call_desktop_api_method("import", params, key)


def import_zip(params: ImportZipParams, key=None) -> None:
    return call_desktop_api_method("importZip", params, key)


def export(params: ExportParams, key=None) -> None:
    return call_desktop_api_method("export", params, key)


def get_files_tree(params: GetFilesTreeParams, key=None) -> List[WorkspaceItem]:
    return call_desktop_api_method("getFilesTree", params, key)


def open(params: OpenParams, key=None) -> None:
    return call_desktop_api_method("open", params, key)


def delete(params: DeleteParams, key=None) -> None:
    return call_desktop_api_method("delete", params, key)


# TODO: Bring back the results listener?
# TODO: No more window.octostar.listen while using the platform-api, cannot do this:
# if st.button("🔍 Manual Entity Search"):
#     searchXperience(key='_manual_entity_link_' + entity_to_rank['contents']['label'])
# search_result = result(key='_manual_entity_link_' + entity_to_rank['contents']['label'])
def searchXperience(params: SearchXperienceParams, key=None) -> List[Entity]:
    result = call_desktop_api_method("searchXperience", params, key)
    return result if not result else [Entity(**v) for v in result]

# TODO: Bring back the results listener? enableSelection won't work without a listener
def get_search_results(params: SearchResultsParams, key=None) -> List[Entity]:
    result = call_desktop_api_method("getSearchResults", params, key)
    return result if not result else [Entity(**v) for v in result]

def show_tab(params: ShowTabParams, key=None) -> None:
    return call_desktop_api_method("showTab", params, key)


def close_tab(params: CloseTabParams, key=None) -> None:
    return call_desktop_api_method("closeTab", params, key)


def call_app_service(params: CallAppServiceParams, key=None) -> None:
    return call_desktop_api_method("callAppService", params, key)


def show_context_menu(
    params: Union[OpenContextMenuParams, CloseContextMenuParams], key=None
) -> None:
    return call_desktop_api_method("showContextMenu", params, key)


def show_toast(params: Union[str, ShowToastParams], key=None) -> None:
    return call_desktop_api_method("showToast", params, key)


def clear_toast(params: ClearToastParams, key=None) -> None:
    return call_desktop_api_method("clearToast", params, key)


def show_progress(params: ShowProgressParams, key=None) -> None:
    return call_desktop_api_method("showProgress", params, key)


def show_confirm(params: Union[str, ShowConfirmParams], key=None) -> Union[bool, None]:
    return call_desktop_api_method("showConfirm", params, key)


def show_file_upload(params: WorkspaceItem, key=None) -> List[WorkspaceItem]:
    return call_desktop_api_method("showFileUpload", params, key)


def show_create_entity_form(
    params: ShowCreateEntityFormParams, key=None
) -> Optional[List[OsWorkspaceEntity]]:
    result = call_desktop_api_method("showCreateEntityForm", params, key)
    if result is None:
        return None
    return [OsWorkspaceEntity(**v) for v in result]


def add_comment(params: AddCommentParams, key=None) -> Optional[Entity]:
    result = call_desktop_api_method("addComment", params, key)
    if result is None:
        return None
    return Entity(**result)


def remove_comment(params: RemoveCommentParams, key=None) -> None:
    return call_desktop_api_method("removeComment", params, key)


def add_watch_intent(params: AddWatchIntentParams, key=None) -> Optional[Entity]:
    result = call_desktop_api_method("addWatchIntent", params, key)
    if result is None:
        return None
    return Entity(**result)


def remove_watch_intent(params: RemoveWatchIntentParams, key=None) -> None:
    return call_desktop_api_method("removeWatchIntent", params, key)


def get_workspace_permission(
    params: GetWorkspacePermissionParams, key=None
) -> Optional[Dict[str, WorkspacePermission]]:
    result = call_desktop_api_method("getWorkspacePermission", params, key)
    if result is None:
        return None
    return {k: WorkspacePermission(**v) for k, v in result.items()}


def get_user(key=None) -> Optional[UserProfile]:
    result = call_desktop_api_method("getUser", None, key)
    if result is None:
        return None
    return UserProfile(**result)


def deploy_app(params: DeployAppParams, key=None) -> WorkspaceItem:
    return call_desktop_api_method("deployApp", params, key)


def undeploy_app(params: UndeployAppParams, key=None) -> WorkspaceItem:
    return call_desktop_api_method("undeployApp", params, key)


def whoami(key=None) -> Optional[Whoami]:
    result = call_desktop_api_method("whoami", None, key)
    if result is None:
        return None
    return Whoami(**result)


# Not implemented methods
# export interface Desktop extends DesktopSettings, WorkspaceChangeEmitter {
#   withProgressBar: <T>(
#     promise: Promise<T>,
#     options?: WithProgressBarOptions,
#   ) => Promise<T>;

#   getPasteContext: (options: PasteContextOptions) => Promise<Entity[]>;
#   showModalTemplate: (props: ModalTemplateProps) => Promise<void>;
# }



from typing import Any, Literal
from octostar_streamlit.core.method_call import MethodCall


DesktopApiMethod = Literal[
    "refresh",
    "openWorkspace",
    "getActiveWorkspace",
    "setActiveWorkspace",
    "closeWorkspace",
    "copy",
    "listAllWorkspaces",
    "getOpenWorkspaces",
    "getOpenWorkspaceIds",
    "setOpenWorkspaceIds",
    "onWorkspaceChanged",
    "onWorkspaceItemChanged",
    "getAttachment",
    "getStylerOptions",
    "getStyler",
    "getWorkspace",
    "getWorkspaceItems",
    "applyTag",
    "removeTag",
    "updateTag",
    "getItem",
    "getItems",
    "getTags",
    "getAvailableTags",
    "getTemplates",
    "getTemplate",
    "getSchemaItems",
    "createWorkspace",
    "connect",
    "save",
    "saveFile",
    "import",
    "importZip",
    "export",
    "getFilesTree",
    "open",
    "delete",
    "searchXperience",
    "getSearchResults",
    "showTab",
    "closeTab",
    "callAppService",
    "showContextMenu",
    "showToast",
    "clearToast",
    "showProgress",
    "withProgressBar",
    "showConfirm",
    "showFileUpload",
    "showCreateEntityForm",
    "getPasteContext",
    "showModalTemplate",
    "addComment",
    "removeComment",
    "addWatchIntent",
    "removeWatchIntent",
    "getWorkspacePermission",
    "getUser",
    "deployApp",
    "undeployApp",
    "whoami",
]


class DesktopMethodCall(MethodCall):
    service: Literal["desktop"] = "desktop"
    method: DesktopApiMethod
    params: Any



import os
import streamlit.components.v1 as components


# Create a _RELEASE constant. We'll set this to False while we're developing
# the component, and True when we're ready to package and distribute it.
# (This is, of course, optional - there are innumerable ways to manage your
# release process.)
_RELEASE = os.environ.get("OCTOSTAR_STREAMLIT_RELEASE", "true").lower() == "true"
# _RELEASE = True

# Declare a Streamlit component. `declare_component` returns a function
# that is used to create instances of the component. We're naming this
# function "_component_func", with an underscore prefix, because we don't want
# to expose it directly to users. Instead, we will create a custom wrapper
# function, below, that will serve as our component's public API.

# It's worth noting that this call to `declare_component` is the
# *only thing* you need to do to create the binding between Streamlit and
# your component frontend. Everything else we do in this file is simply a
# best practice.
component_name = "octostar_streamlit_comunication_bridge"

if not _RELEASE:
    _component_func = components.declare_component(
        # We give the component a simple, descriptive name ("my_component"
        # does not fit this bill, so please choose something better for your
        # own component :)
        component_name,
        # Pass `url` here to tell Streamlit that the component will be served
        # by the local dev server that you run via `npm run start`.
        # (This is useful while your component is in development.)
        url="http://localhost:5173",
    )
else:
    # When we're distributing a production version of the component, we'll
    # replace the `url` param with `path`, and point it to the component's
    # build directory:
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    build_dir = os.path.join(parent_dir, "frontend/dist")
    _component_func = components.declare_component(component_name, path=build_dir)



from typing import Any, Literal

from octostar_streamlit.core.method_call import MethodCall


ExtrasApiMethod = Literal["createLinkChart"]


class ExtrasMethodCall(MethodCall):
    service: Literal["extras"] = "extras"
    method: ExtrasApiMethod
    params: Any



from typing import Any, Literal
from octostar_streamlit.core.method_call import MethodCall

OntologyApiMethod = Literal[
    "cancelQueries",
    "getAvailableOntologies",
    "getWorkspaceRelationshipRecords",
    "clearRelationshipCache",
    "getConnectedEntities",
    "getConceptByName",
    "getConcepts",
    "getEntity",
    "getOntologyName",
    "getRelationshipCount",
    "getConceptForEntity",
    "getRelationshipsForEntity",
    "sendQuery",
    "sendQueryT",
    "getSysInheritance",
    "subscribe",
    "consistentUUID",
]


class OntologyMethodCall(MethodCall):
    service: Literal["ontology"] = "ontology"
    method: OntologyApiMethod
    params: Any



from typing import Union

from octostar_streamlit.core.entities import Entity
from octostar_streamlit.core.params_base_model import ParamsBaseModel

class SavedSearchIdn(ParamsBaseModel):
    entity: Union[str, Entity]
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        if isinstance(self.entity, Entity):
            base_dict['entity'] = self.entity.to_dict()
        return base_dict



from typing import Any, List, Optional

from octostar_streamlit.core.components.method_call import RenderComponentCall, Components
from octostar_streamlit.core.components.params import OsContextMenuParams, OsDropzoneParams
from octostar_streamlit.core.desktop.params import SearchXperienceParams
from octostar_streamlit.core.entities import Entity
from octostar_streamlit.core.streamlit_executor import StreamlitMethodExecutor
from octostar_streamlit import _component_func


def call_components_api_method(method: Components, params: Any, key=None):
    call = RenderComponentCall(service="component", method=method, params=params)
    value = StreamlitMethodExecutor(
        method_call=call, fn=_component_func, key=key
    ).execute()

    return value

def os_contextmenu(params: OsContextMenuParams, key=None) -> None:
    return call_components_api_method("OsContextMenu", params, key=key)

def os_dropzone(params: OsDropzoneParams, key=None) -> Optional[List[dict]]:
    return call_components_api_method("OsDropzone", params, key=key)

def os_searchbutton(params: SearchXperienceParams, key=None) -> Optional[List[Entity]]:
    result = call_components_api_method("SearchXperience", params, key)
    return result if not result else [Entity(**v) for v in result]



# from humps import camelize
from pydantic import BaseModel, ConfigDict


class ParamsBaseModel(BaseModel):
    model_config = ConfigDict(
        arbitrary_types_allowed=True,
    )
    # class Config:
    #     arbitrary_types_allowed = True
    # #     alias_generator = camelize
    # #     populate_by_name = True



from typing import List, Union

from octostar_streamlit.core.entities import EdgeSpec, Entity
from octostar_streamlit.core.params_base_model import ParamsBaseModel

# TODO: Typing for subscribeToChanges params


"""
Entities accepted or returned by the desktop API methods.

In most cases the entities are dataclasses so they have the following properties:
- Can be constructed with keyword arguments without pydantic validation;
- When used with pydantic BaseModel become validated models (default behavior of pydantic with stdlib dataclasses);
More: https://docs.pydantic.dev/latest/concepts/dataclasses/.

Occasionally entities are subclasses of ParamsBaseModel.
It is to cover the cases when we need to pass an entity as a parameter to the desktop API method.
"""
from dataclasses import dataclass, field
from typing import Any, Dict, List, Literal, Union
from octostar_streamlit.core.params_base_model import ParamsBaseModel

class WorkspaceIdHost(ParamsBaseModel):
    id: str

# TODO: Look into consolidating this and WorkspaceItemModel
WorkspaceItem = Dict

WorkspaceItems = List[WorkspaceItem]

WorkspaceIdentifier = Union[WorkspaceIdHost, WorkspaceItem]


# TODO: Look into consolidating Entity and BaseEntityModel
# We need the entities returned from the SDK to be able to funnel back into let’s say createLinkChart in a lossless fashion.
@dataclass(init=False)
class Entity:
    entity_id: str
    entity_type: str
    entity_label: str
    additional_properties: dict = field(default_factory=dict)

    def __init__(self, entity_id: str, entity_type: str, entity_label: str, **kwargs):
        self.entity_id = entity_id
        self.entity_type = entity_type
        self.entity_label = entity_label
        # Capture any additional fields
        self.additional_properties = kwargs

    def to_dict(self):
        # Convert the instance back to a dictionary and flatten additional properties
        base_dict = {
            "entity_id": self.entity_id,
            "entity_type": self.entity_type,
            "entity_label": self.entity_label,
        }
        return {**base_dict, **self.additional_properties}

OsTag = Dict


@dataclass
class TagInfo:
    tag: str
    os_workspace: str
    count: int
    sample: List[Entity]
    entity: OsTag


@dataclass
class WorkspaceRecordInfo:
    concept: str
    count: int
    max_last_updated: str
    entities: List[Entity]


WorkspaceRecords = Dict[str, WorkspaceRecordInfo]


# This class is used as enum
class WorkspacePermissionValue:
    NONE = 0
    Read = 1
    Write = 2
    Admin = 4


@dataclass
class WorkspacePermission:
    value: WorkspacePermissionValue
    label: str


@dataclass
class Workspace:
    workspace: WorkspaceItem
    items: WorkspaceItems
    os_flags: int
    workspace_records: Union[WorkspaceRecords, None] = None
    tags: Union[List[TagInfo], None] = None
    permission: Union[WorkspacePermission, None] = None
    # FIXME: consider implementing automaticacasting to snake_case
    isActive: Union[bool, None] = None


@dataclass
class Colorful:
    color: Union[str, None] = None


@dataclass
class RequiredTagAttributes:
    os_workspace: str
    os_item_name: str


@dataclass
class TagAttributes(Colorful, RequiredTagAttributes):
    # os_workspace: str
    # os_item_name: str
    group: Union[str, None] = None
    order: Union[int, None] = None


AttachmentType = Any

AttachmentResponseType = Literal["text", "blob", "arrayBuffer", "json", "url"]


@dataclass
class StylerOption:
    name: str
    description: str


Styler = Dict

TemplateType = Literal["nunjucks", "javascript"]

TemplateVariant = Literal["antd"]


@dataclass
class TemplateMetadataRequiredFields:
    name: str
    description: str
    concepts: List[str]
    type: TemplateType
    variant: TemplateVariant
    parameterNames: List[str]


@dataclass
class TemplateMetadataOptionalFields:
    blurhash: Union[str, None] = None
    style: Union[Dict, None] = None
    rendersMultipleRecords: Union[bool, None] = None


@dataclass
class TemplateMetadata(TemplateMetadataOptionalFields, TemplateMetadataRequiredFields):
    pass


@dataclass
class CustomTemplateRequiredFields:
    template: str


@dataclass
class CustomTemplate(TemplateMetadata, CustomTemplateRequiredFields):
    pass


@dataclass
class Relationship:
    key: str
    concept: str
    target_concept: str
    relationship_name: str
    inverse_name: str
    source_properties: str
    target_properties: str
    mapping_name: str
    additional_properties: str
    mapping_query: str
    tables: str
    mapping_json: str
    is_inverse: int
    is_mtm: int
    transitivity: int
    datasource_id: str
    sort_order: int
    description: Union[str, None] = None


@dataclass
class WorkspaceRecordIdentifier:
    entity_type: str
    os_entity_uid: str
    os_workspace: str


@dataclass
class WorkspaceRecordWithRelationships:
    entity: WorkspaceItem
    relationship_name: Union[str, None] = None
    relationships: Union[List[Dict], None] = None


EdgeSpec = Dict


@dataclass
class GraphSpec:
    nodes: List[Entity]
    edges: List[EdgeSpec]
    workspace_item: Union[Entity, None] = None


IDataTransfer = Dict


ContextMenuGroup = Literal[
    "add",
    "addto",
    "edit",
    "transform",
    "general",
    "open",
    "danger",
    "tags",
]

ContextMenuLabels = Dict[ContextMenuGroup, str]

ItemType = Dict


ButtonProps = Dict


class OsWorkspaceEntity(Entity, WorkspaceRecordIdentifier):
    pass


# [key: string, value: any]
KeyValueTuple = List


@dataclass
class UserProfile:
    firstName: str
    lastName: str
    email: str
    username: str


@dataclass
class Whoami:
    async_channel: str
    os_jwt: str
    username: str
    is_superuser: bool
    roles: List[str]
    timbr_roles: List[str]
    email: Union[str, None] = None


@dataclass
class OsAuditFields:
    os_created_by: Union[str, None] = None
    os_created_at: Union[str, None] = None
    os_deleted_by: Union[str, None] = None
    os_deleted_at: Union[str, None] = None
    os_hidden_by: Union[str, None] = None
    os_hidden_at: Union[str, None] = None
    os_last_updated_by: Union[str, None] = None
    os_last_updated_at: Union[str, None] = None


@dataclass
class WorkspaceItemModel:
    os_item_name: str
    os_item_type: str


@dataclass
class WorkspaceItemBase(WorkspaceItemModel):
    os_entity_uid: str
    os_workspace: str


@dataclass
class WorkspaceRelationshipRequiredFields:
    os_entity_uid_from: str
    os_entity_uid_to: str
    os_entity_type_from: str
    os_entity_type_to: str
    os_relationship_name: str


@dataclass
class WorkspaceRelationship(
    OsAuditFields, WorkspaceRelationshipRequiredFields, WorkspaceItemBase
):
    pass


@dataclass
class Property:
    concept: str
    property_name: str
    property_type: str


@dataclass
class Tag:
    tag_name: str
    tag_value: str


@dataclass
class Concept:
    concept_name: str
    parents: List[str]
    relationships: List[Relationship]
    columns: List[str]
    archetype: Dict
    tags: List[Tag]
    labelKeys: List[str]
    allLabelKeys: List[str]
    properties: Union[List[Property], None] = None
    allProperties: Union[List[Property], None] = None


@dataclass
class RelationshipCountResult:
    count: int
    expired: Union[bool, None] = None


@dataclass
class Inheritance:
    base_concept: str
    derived_concept: str



from typing import Any

# from octostar_streamlit.core.extras.method_call import ExtrasApiMethod, ExtrasMethodCall
# from octostar_streamlit.core.extras.params import CreateLinkChartParams, OsDropzoneParams
from octostar_streamlit.core.context.method_call import ContextMethodCall, ContextApiMethod
from octostar_streamlit.core.streamlit_executor import StreamlitMethodExecutor
from octostar_streamlit import _component_func


def call_context_api_method(method: ContextApiMethod, params: Any, key=None):
    call = ContextMethodCall(service="context", method=method, params=params)
    value = StreamlitMethodExecutor(
        method_call=call, fn=_component_func, key=key
    ).execute()

    return value

def get_context(key=None):
    return call_context_api_method("getContext", {}, key=key)

# TODO: Decide their fate:
def subscribe_to_changes(id: str, callback):
    raise Exception("Function not implemented.")

def unsubscribe_from_changes(id: str):
    raise Exception("Function not implemented.")


from typing import Any, Literal
from octostar_streamlit.core.method_call import MethodCall


Components = Literal[
    "SearchXperience",
    "OsDropzone",
    "OsContextMenu",
]

class RenderComponentCall(MethodCall):
    service: Literal["component"] = "component"
    method: Components
    params: Any



from typing import Any, Literal

from octostar_streamlit.core.method_call import MethodCall


RemoteAppApiMethod = Literal[
    "subscribeToContext",
    "unsubscribeFromContext",
    "getCurrentWorkspaceId",
    "subscribeToDragStart",
    # "unsubscribeFromDragStart",
    # "dropZoneRequest",
    "setTransformResult",
]


class RemoteAppMethodCall(MethodCall):
    service: Literal["remoteApp"] = "remoteApp"
    method: RemoteAppApiMethod
    params: Any



from typing import Dict, List, Union

from octostar_streamlit.core.entities import EdgeSpec, Entity
from octostar_streamlit.core.params_base_model import ParamsBaseModel


class CreateLinkChartParams(ParamsBaseModel):
    name: str
    path: Union[str, None] = None
    # TODO: Test if Entity with additional_properties thing would be okay?
    nodes: Union[List[Entity], None] = None
    edges: Union[List[EdgeSpec], None] = None
    draft: Union[bool, None] = None
    os_workspace: Union[str, None] = None

    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        if self.nodes is not None:
            base_dict['nodes'] = [record.to_dict() for record in self.nodes]
        return base_dict



from typing import Any, Literal

from octostar_streamlit.core.method_call import MethodCall


SavedSearchApiMethod = Literal["getRecordsCount"]


class SavedSearchMethodCall(MethodCall):
    service: Literal["savedSearch"] = "savedSearch"
    method: SavedSearchApiMethod
    params: Any



from octostar_streamlit.core.params_base_model import ParamsBaseModel


class SubscribeToContextParams(ParamsBaseModel):
    id: str


class UbsubscribeFromContextParams(ParamsBaseModel):
    id: str



from dataclasses import dataclass
from typing import Union
from octostar_streamlit.core.entities import Entity, Relationship
from octostar_streamlit.core.params_base_model import ParamsBaseModel


class CancelQueriesParams(ParamsBaseModel):
    context: str


class GetWorkspaceRelationshipRecordsParams(ParamsBaseModel):
    entity: Entity
    relationship: Union[str, Relationship]
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict

class ClearRelationshipCacheParams(ParamsBaseModel):
    entity: Entity
    relationship: Union[str, Relationship]
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict

class GetConnectedEntitiesParams(ParamsBaseModel):
    entity: Entity
    relationship: Union[str, Relationship]
    forceRefresh: Union[
        bool, None
    ]  # TODO: add alias to automatically convert to camelCase instead?
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict

class GetConceptByNameParams(ParamsBaseModel):
    conceptName: str  # TODO: add alias to automatically convert to camelCase instead?


class GetEntityParams(ParamsBaseModel):
    entity: Entity
    refresh: Union[bool, None] = None
    skipSideEffects: Union[bool, None] = (
        None  # TODO: add alias to automatically convert to camelCase instead?
    )
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict


class GetRelationshipCountParams(ParamsBaseModel):
    entity: Entity
    relationship: Union[str, Relationship]
    forceRefresh: Union[
        bool, None
    ]  # TODO: add alias to automatically convert to camelCase instead?
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict


class GetConceptForEntityparams(ParamsBaseModel):
    entity: Entity
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict


class GetRelationshipsForEntityParams(ParamsBaseModel):
    entity: Entity
    def model_dump(self, **kwargs):
        base_dict = super().model_dump(**kwargs)
        base_dict['entity'] = self.entity.to_dict()
        return base_dict


@dataclass
class SendQueryOptions:
    context: Union[str, None] = None
    low_priority: Union[bool, None] = None


class SendQueryParams(ParamsBaseModel):
    query: str
    options: Union[SendQueryOptions, None] = None

class SendQueryTParams(SendQueryParams):
    force_refresh: Union[bool, None] = None


class ConsistentUUIDParams(ParamsBaseModel):
    name: str
    namespace: Union[str, None] = None



from typing import Any, Dict

from octostar_streamlit import _component_func
from octostar_streamlit.core.remote_app.method_call import (
    RemoteAppApiMethod,
    RemoteAppMethodCall,
)
from octostar_streamlit.core.remote_app.params import (
    SubscribeToContextParams,
    UbsubscribeFromContextParams,
)
from octostar_streamlit.core.streamlit_executor import StreamlitMethodExecutor


def call_remote_app_api_method(method: RemoteAppApiMethod, params: Any, key=None):
    call = RemoteAppMethodCall(service="remoteApp", method=method, params=params)
    value = StreamlitMethodExecutor(
        method_call=call, fn=_component_func, key=key
    ).execute()

    return value


def subscribe_to_context(id: str, key=None) -> Dict:
    if key is None:
        key = id

    return call_remote_app_api_method(
        "subscribeToContext", params=SubscribeToContextParams(id=id), key=key
    )


def unsubscribe_from_context(id: str, key=None) -> None:
    return call_remote_app_api_method(
        "unsubscribeFromContext", params=UbsubscribeFromContextParams(id=id), key=key
    )


def set_transform_result(params: Any, key=None) -> None:
    return call_remote_app_api_method("setTransformResult", params=params, key=key)

def get_current_workspace_id(key=None) -> str:
    return call_remote_app_api_method("getCurrentWorkspaceId", params=None, key=key)


import http.server
import socketserver
import time

class StallRequestHandler(http.server.SimpleHTTPRequestHandler):
    def do_GET(self):
        self.send_response(200)  # Optional: Send a response header if you want partial success
        self.end_headers()
        while True:
            time.sleep(1)  # Keep the connection alive forever

PORT = 8501
with socketserver.TCPServer(("", PORT), StallRequestHandler) as httpd:
    print(f"Stalling server running on port {PORT}")
    httpd.serve_forever()
